use eframe::egui::{self, Response, Ui, Widget};

use super::state::AppState;

pub struct Leaderboard {
    entries: Vec<LeaderboardEntry>,
}

#[derive(Clone)]
struct LeaderboardEntry {
    name: String,
    tokens: i32,
    time: f32,
}

impl Leaderboard {
    pub fn new(app_state: &AppState) -> Self {
        let entries = app_state
            .selected_models
            .iter()
            .map(|model| LeaderboardEntry {
                name: model.name.clone(),
                tokens: model.tokens,
                time: model.time,
            })
            .collect();

        Self { entries }
    }
}

impl Widget for Leaderboard {
    fn ui(self, ui: &mut Ui) -> Response {
        let leaderboard_gap = 8.0;

        let mut time_leaderboard = self.entries.clone();
        time_leaderboard.sort_by(|a, b| {
            a.time
                .partial_cmp(&b.time)
                .unwrap_or(std::cmp::Ordering::Equal)
        });

        let mut tokens_leaderboard = self.entries.clone();
        tokens_leaderboard.sort_by(|a, b| {
            b.tokens
                .partial_cmp(&a.tokens)
                .unwrap_or(std::cmp::Ordering::Equal)
        });

        let mut time_per_token_leaderboard = self.entries.clone();
        time_per_token_leaderboard.sort_by(|a, b| {
            (a.time / a.tokens as f32)
                .partial_cmp(&(b.time / b.tokens as f32))
                .unwrap_or(std::cmp::Ordering::Equal)
        });

        ui.vertical(|ui| {
            ui.heading("Leaderboard");

            ui.horizontal_top(|ui| {
                egui::Grid::new("time_leaderboard")
                    .striped(true)
                    .show(ui, |ui| {
                        ui.strong("Name");
                        ui.strong("Time");
                        ui.end_row();

                        for entry in time_leaderboard {
                            ui.label(entry.name);
                            ui.label(entry.time.to_string());
                            ui.end_row();
                        }
                    });

                ui.allocate_space(egui::Vec2::new(leaderboard_gap, 0.0));

                egui::Grid::new("tokens_leaderboard")
                    .striped(true)
                    .show(ui, |ui| {
                        ui.strong("Name");
                        ui.strong("Tokens");
                        ui.end_row();

                        for entry in tokens_leaderboard {
                            ui.label(entry.name);
                            ui.label(entry.tokens.to_string());
                            ui.end_row();
                        }
                    });

                ui.allocate_space(egui::Vec2::new(leaderboard_gap, 0.0));

                egui::Grid::new("time_per_token_leaderboard")
                    .striped(true)
                    .show(ui, |ui| {
                        ui.strong("Name");
                        ui.strong("Time per Token");
                        ui.end_row();

                        for entry in time_per_token_leaderboard {
                            ui.label(entry.name);
                            ui.label((entry.time / entry.tokens as f32).to_string());
                            ui.end_row();
                        }
                    });
            })
        })
        .response
    }
}
