use eframe::{
    egui::{self, Layout, Separator},
    CreationContext,
};

use super::{leaderboard, model_response::ModelResponse, model_selection, prompt_input};

pub struct AppState {
    pub selected_models: Vec<ModelResponse>,
    pub prompt_input: String,
    pub models_available: Vec<String>,
}

impl AppState {
    pub fn new(_cc: &CreationContext<'_>, models_available: Vec<String>) -> Self {
        Self {
            selected_models: vec![],
            prompt_input: String::new(),
            models_available,
        }
    }

    pub fn handle_submission(&mut self) {
        for model in &mut self.selected_models {
            model.start_chat_completion(self.prompt_input.clone());
        }
    }
}

impl eframe::App for AppState {
    fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
        egui::CentralPanel::default().show(ctx, |ui| {
            ui.add(model_selection::ModelSelection::new(self));

            ui.allocate_space(egui::Vec2::new(0.0, 8.0));

            ui.horizontal(|ui| {
                ui.add(prompt_input::PromptInput::new(self));
                ui.allocate_space(egui::Vec2::new(16.0, 0.0));
                ui.add(leaderboard::Leaderboard::new(self));
            });
            

            ui.add(Separator::default().spacing(16.0));

            let size = ui.available_size_before_wrap();
            let widget_size = egui::Vec2 {
                x: size.x / 4.0,
                y: size.y,
            };

            ui.allocate_ui_with_layout(size, Layout::left_to_right(egui::Align::Min), |ui| {
                egui::ScrollArea::both().show(ui, |ui| {
                    ui.horizontal_top(|ui| {
                        for model in &mut self.selected_models {
                            ui.add_sized(widget_size, model);
                        }
                    });
                });
            });
        });
    }
}
